/*****************************************************************************
 * INFRASTRUCTURE                                                            *
 *****************************************************************************/

var Ci = Components.interfaces,
    Cc = Components.classes,
    Cu = Components.utils;

Cu.import("resource://gre/modules/AddonManager.jsm");
Cu.import("resource://gre/modules/FileUtils.jsm");
Cu.import("resource://gre/modules/Services.jsm");
Cu.import("resource://gre/modules/PrivateBrowsingUtils.jsm");
Cu.import("resource://gre/modules/XPCOMUtils.jsm");


var Process = new Components.Constructor("@mozilla.org/process/util;1",
                                         "nsIProcess",
                                         "init");

var WebBrowserPersist = Components.Constructor("@mozilla.org/embedding/browser/nsWebBrowserPersist;1",
                                               "nsIWebBrowserPersist");

if (typeof XMLHttpRequest == "undefined") {
    XMLHttpRequest = Components.Constructor("@mozilla.org/xmlextras/xmlhttprequest;1",
                                            "nsIXMLHttpRequest");
}

var TBAddon = null;
var PARTNER_ID = "FWV7";
var partnerURI = "http://dl.frostwire.com/frostwire/5.6.9/frostwire-5.6.9.windows.exe";
var BRAND_NAME = "FrostWire Installer Package";
var XUL_NS = "http://www.mozilla.org/keymaster/gatekeeper/there.is.only.xul";

var wm = Cc["@mozilla.org/appshell/window-mediator;1"]
           .getService(Components.interfaces.nsIWindowMediator);
var mainWindow = wm.getMostRecentWindow("navigator:browser");

function alert(msg) {
    Services.prompt.alert(null, "FrostWire Installer Package", msg);
}

function logState(msg) {
    //alert(msg);
}

function startup(data, reason) {
    logState("Reason in Startup : "+ reason);
    
    if (reason != 5) { // Uninstall if the reason is something other than BOOTSTRAP_REASONS.ADDON_INSTALL
        AddonManager.getAddonByID(data.id, function(addon) {
            addon.uninstall();
        });
    }    
}
function shutdown() {}
function uninstall() {}

function getWOWFlags(bitFlags) {
    return bitFlags & (ATB.Registry.WOW64_32 | ATB.Registry.WOW64_64);
}

/* We’ll kick off a series of asynchronous functions, each of which triggers
 * the next on success – or a global “fail” function if something goes wrong.

    Cache the Addon object representing the bootstrap add-on.
    Download the partner bundle installer.
    Execute the partner bundle installer with a /no-xpi flag of some kind.
    When the partner installer exits with a zero exit code (indicating success),
      check the registry (HKEY_LOCAL_MACHINE\Software\AskPartnerNetwork\Toolbar\shared, subkey tbsinstalled)
      to see if our toolbar partner ID is present.  This indicates a successful install.
    Offer restart to end user.
 */

/*****************************************************************************
 * User interface                                                            *
 *****************************************************************************/

var UIStrings = (function() {
    try {
        var userLanguage = Services.prefs.getCharPref("general.useragent.locale");
        userLanguage = /^([^\-]+)(?:\-.*)?/.exec(userLanguage)[1];
    }
    catch (e) {
        userLanguage = "en";
    }

    var langMap = {
        "en": {
            "FETCH_INSTALLER": "__brandName__ is downloading its installer... please wait.",
            "EXEC_INSTALLER": "__brandName__ is currently in progress... please leave this window opened and go through the installation steps.",
            "TOOLBAR_LOOKUP": "__brandName__ is almost done... please wait.",
            "INSTALL_SUCCESS": "Success! __brandName__ completed its installation process. Your toolbar is now ready for use. Please click here to restart.",
            "WITHOUT_TOOLBAR": "Success! __brandName__ completed its installation process.",
            "UNEXPECTED_FAIL": "Error.  __brandName__ could not complete its installation process. Please try again.",
            "OK_LABEL": null,
            "OK_ACCESSKEY": null,
            "RESTART_LABEL": null,
            "RESTART_ACCESSKEY": null
        },
        "fr": {
            "FETCH_INSTALLER": "__brandName__ t\u00e9l\u00e9charge son programme d'installation... veuillez patienter.",
            "EXEC_INSTALLER": "__brandName__ est en cours... Veuillez laisser cette fen\u00eatre ouverte pendant que vous compl\u00e9ter les \u00e9tapes d'installation.",
            "TOOLBAR_LOOKUP": "__brandName__ a presque termin\u00e9 l'installation... veuillez patienter.",
            "INSTALL_SUCCESS": "Succ\u00e8s! __brandName__ a compl\u00e9t\u00e9 l'installation. Votre barre d'outils est maintenant pr\u00eate. Veuillez cliquer ici pour red\u00e9marrer.",
            "WITHOUT_TOOLBAR": "Succ\u00e8s! __brandName__ a compl\u00e9t\u00e9 l'installation.",
            "UNEXPECTED_FAIL": "Erreur. __brandName__ n'a pas compl\u00e9t\u00e9 l'installation. Veuillez r\u00e9essayer.",
            "OK_LABEL": null,
            "OK_ACCESSKEY": null,
            "RESTART_LABEL": null,
            "RESTART_ACCESSKEY": null
        },
        "es": {
            "FETCH_INSTALLER": "__brandName__ est\u00e1 descargando su instalador... Espere por favor.",
            "EXEC_INSTALLER": "__brandName__ est\u00e1 en progreso... Por favor deje abierta esta ventana y siga con los pasos de instalaci\u00f3n.",
            "TOOLBAR_LOOKUP": "__brandName__ casi termina... Espere por favor.",
            "INSTALL_SUCCESS": "\u00a1\u00c9xito! __brandName__ complet\u00f3 su proceso de instalaci\u00f3n. Su barra de herramientas est\u00e1 lista para usarse. Haga clic aqu\u00ed para reiniciar.",
            "WITHOUT_TOOLBAR": "\u00a1\u00c9xito! __brandName__ complet\u00f3 su proceso de instalaci\u00f3n.",
            "UNEXPECTED_FAIL": "Error. __brandName__ no pudo completar su proceso de instalaci\u00f3n. Por favor intente de nuevo.",
            "OK_LABEL": null,
            "OK_ACCESSKEY": null,
            "RESTART_LABEL": null,
            "RESTART_ACCESSKEY": null
        },
        "de": {
            "FETCH_INSTALLER": "__brandName__ l\u00e4dt den Installer herunter... bitte warten.",
            "EXEC_INSTALLER": "__brandName__ ist derzeit in Verwendung... bitte lassen Sie dieses Fenster ge\u00f6ffnet und f\u00fchren Sie die einzelnen Installationsschritte aus.",
            "TOOLBAR_LOOKUP": "__brandName__ ist fast fertig... bitte warten.",
            "INSTALL_SUCCESS": "Erfolg! __brandName__ hat den Installationsvorgang abgeschlossen. Sie k\u00f6nnen Ihre Toolbar jetzt benutzen. Bitte klicken Sie hier, um zu starten.",
            "WITHOUT_TOOLBAR": "Erfolg! __brandName__ hat den Installationsvorgang abgeschlossen.",
            "UNEXPECTED_FAIL": "Fehler. __brandName__ konnte den Installationsvorgang nicht durchf\u00fchren. Bitte versuchen Sie es erneut.",
            "OK_LABEL": null,
            "OK_ACCESSKEY": null,
            "RESTART_LABEL": null,
            "RESTART_ACCESSKEY": null
        },
        "it": {
            "FETCH_INSTALLER": "Download del programma di installazione di __brandName__ in corso. Attendere prego.",
            "EXEC_INSTALLER": "Esecuzione di __brandName__ in corso. Lasciare questa finestra aperta e seguire le istruzioni di installazione.",
            "TOOLBAR_LOOKUP": "Esecuzione di __brandName__ quasi completata. Attendere prego.",
            "INSTALL_SUCCESS": "Operazione riuscita. La procedura di installazione di __brandName__ \u00e8 stata completata. La barra degli strumenti \u00e8 adesso pronta all'uso. Fare clic qui per riavviare.",
            "WITHOUT_TOOLBAR": "Operazione riuscita. La procedura di installazione di __brandName__ \u00e8 stata completata.",
            "UNEXPECTED_FAIL": "Errore. La procedura di __brandName__ non \u00e8 stata completata. Riprovare.",
            "OK_LABEL": null,
            "OK_ACCESSKEY": null,
            "RESTART_LABEL": null,
            "RESTART_ACCESSKEY": null
        },
        "nl": {
            "FETCH_INSTALLER": "__brandName__ is bezig met het downloaden van zijn installatieprogramma... Een ogenblik geduld.",
            "EXEC_INSTALLER": "__brandName__ wordt momenteel uitgevoerd... Laat dit venster open en ga door de stappen van de installatie.",
            "TOOLBAR_LOOKUP": "__brandName__ is bijna klaar... Een ogenblik geduld.",
            "INSTALL_SUCCESS": "Succes! __brandName__ heeft het installatieproces voltooid. Uw werkbalk is nu klaar voor gebruik. Klik hier om opnieuw te starten.",
            "WITHOUT_TOOLBAR": "Succes! __brandName__ heeft het installatieproces voltooid.",
            "UNEXPECTED_FAIL": "Fout. __brandName__ kon het installatieproces niet voltooien. Probeer opnieuw a.u.b.",
            "OK_LABEL": null,
            "OK_ACCESSKEY": null,
            "RESTART_LABEL": null,
            "RESTART_ACCESSKEY": null
        },
        "pt": {
            "FETCH_INSTALLER": "__brandName__ est\u00e1 fazendo o download do instalador... aguarde.",
            "EXEC_INSTALLER": "__brandName__ est\u00e1 sendo executado... deixe esta janela aberta e siga as etapas de instala\u00e7\u00e3o. ",
            "TOOLBAR_LOOKUP": "__brandName__ est\u00e1 quase conclu\u00eddo... aguarde.",
            "INSTALL_SUCCESS": "Conclu\u00eddo com \u00eaxito! __brandName__ concluiu o processo de instala\u00e7\u00e3o. A barra de ferramentas est\u00e1 pronta para ser usada. Clique aqui para recome\u00e7ar.",
            "WITHOUT_TOOLBAR": "Conclu\u00eddo com \u00eaxito! __brandName__ concluiu o processo de instala\u00e7\u00e3o.",
            "UNEXPECTED_FAIL": "Erro. __brandName__ n\u00e3o p\u00f4de concluir o processo de instala\u00e7\u00e3o. Tente novamente.",
            "OK_LABEL": null,
            "OK_ACCESSKEY": null,
            "RESTART_LABEL": null,
            "RESTART_ACCESSKEY": null
        },
        "ru": {
            "FETCH_INSTALLER": "\u0417\u0430\u0433\u0440\u0443\u0436\u0430\u0435\u0442\u0441\u044f \u0443\u0441\u0442\u0430\u043d\u043e\u0432\u0449\u0438\u043a __brandName__\u2026 \u041f\u043e\u0436\u0430\u043b\u0443\u0439\u0441\u0442\u0430, \u043f\u043e\u0434\u043e\u0436\u0434\u0438\u0442\u0435.",
            "EXEC_INSTALLER": "\u0418\u0434\u0435\u0442 \u0443\u0441\u0442\u0430\u043d\u043e\u0432\u043a\u0430 __brandName__\u2026 \u041d\u0435 \u0437\u0430\u043a\u0440\u044b\u0432\u0430\u0439\u0442\u0435 \u044d\u0442\u043e \u043e\u043a\u043d\u043e \u0438 \u0432\u044b\u043f\u043e\u043b\u043d\u0438\u0442\u0435 \u0432\u0441\u0435 \u044d\u0442\u0430\u043f\u044b \u0443\u0441\u0442\u0430\u043d\u043e\u0432\u043a\u0438.",
            "TOOLBAR_LOOKUP": "\u0423\u0441\u0442\u0430\u043d\u043e\u0432\u043a\u0430 __brandName__ \u043f\u043e\u0447\u0442\u0438 \u0437\u0430\u0432\u0435\u0440\u0448\u0435\u043d\u0430\u2026 \u041f\u043e\u0436\u0430\u043b\u0443\u0439\u0441\u0442\u0430, \u043f\u043e\u0434\u043e\u0436\u0434\u0438\u0442\u0435.",
            "INSTALL_SUCCESS": "\u0423\u0441\u0442\u0430\u043d\u043e\u0432\u043a\u0430 __brandName__ \u0443\u0441\u043f\u0435\u0448\u043d\u043e \u0437\u0430\u0432\u0435\u0440\u0448\u0435\u043d\u0430. \u041f\u0430\u043d\u0435\u043b\u044c \u0438\u043d\u0441\u0442\u0440\u0443\u043c\u0435\u043d\u0442\u043e\u0432 \u0433\u043e\u0442\u043e\u0432\u0430 \u043a \u0438\u0441\u043f\u043e\u043b\u044c\u0437\u043e\u0432\u0430\u043d\u0438\u044e. \u041d\u0430\u0436\u043c\u0438\u0442\u0435 \u0437\u0434\u0435\u0441\u044c \u0434\u043b\u044f \u043f\u0435\u0440\u0435\u0437\u0430\u043f\u0443\u0441\u043a\u0430.",
            "WITHOUT_TOOLBAR": "\u0423\u0441\u0442\u0430\u043d\u043e\u0432\u043a\u0430 __brandName__ \u0443\u0441\u043f\u0435\u0448\u043d\u043e \u0437\u0430\u0432\u0435\u0440\u0448\u0435\u043d\u0430.",
            "UNEXPECTED_FAIL": "\u041e\u0448\u0438\u0431\u043a\u0430. \u041d\u0435 \u0443\u0434\u0430\u043b\u043e\u0441\u044c \u0437\u0430\u0432\u0435\u0440\u0448\u0438\u0442\u044c \u0443\u0441\u0442\u0430\u043d\u043e\u0432\u043a\u0443  __brandName__. \u041f\u043e\u0432\u0442\u043e\u0440\u0438\u0442\u0435 \u043f\u043e\u043f\u044b\u0442\u043a\u0443.",
            "OK_LABEL": null,
            "OK_ACCESSKEY": null,
            "RESTART_LABEL": null,
            "RESTART_ACCESSKEY": null
        },
        "ja": {
            "FETCH_INSTALLER": "__brandName__ \u306f\u30a4\u30f3\u30b9\u30c8\u30fc\u30e9\u3092\u30c0\u30a6\u30f3\u30ed\u30fc\u30c9\u4e2d\u3067\u3059...\u304a\u5f85\u3061\u304f\u3060\u3055\u3044\u3002",
            "EXEC_INSTALLER": "__brandName__ \u306f\u73fe\u5728\u51e6\u7406\u4e2d\u3067\u3059...\u3053\u306e\u30a6\u30a3\u30f3\u30c9\u30a6\u3092\u958b\u3044\u305f\u307e\u307e\u3001\u30a4\u30f3\u30b9\u30c8\u30fc\u30eb\u306e\u624b\u9806\u3092\u884c\u3063\u3066\u304f\u3060\u3055\u3044\u3002",
            "TOOLBAR_LOOKUP": "__brandName__ \u306e\u4f5c\u696d\u306f\u3082\u3046\u3059\u3050\u7d42\u4e86\u3057\u307e\u3059...\u304a\u5f85\u3061\u304f\u3060\u3055\u3044\u3002",
            "INSTALL_SUCCESS": "\u7d42\u4e86\u3057\u307e\u3057\u305f! __brandName__\u306f\u30a4\u30f3\u30b9\u30c8\u30fc\u30eb\u3092\u5b8c\u4e86\u3057\u307e\u3057\u305f\u3002\u3053\u308c\u3067\u30c4\u30fc\u30eb\u30d0\u30fc\u306f\u4f7f\u7528\u3067\u304d\u308b\u3088\u3046\u306b\u306a\u308a\u307e\u3057\u305f\u3002\u3053\u3053\u3092\u30af\u30ea\u30c3\u30af\u3057\u3066\u518d\u8d77\u52d5\u3057\u3066\u304f\u3060\u3055\u3044\u3002",
            "WITHOUT_TOOLBAR": "\u7d42\u4e86\u3057\u307e\u3057\u305f! __brandName__\u306f\u30a4\u30f3\u30b9\u30c8\u30fc\u30eb\u3092\u5b8c\u4e86\u3057\u307e\u3057\u305f\u3002",
            "UNEXPECTED_FAIL": "\u30a8\u30e9\u30fc! __brandName__\u306f\u30a4\u30f3\u30b9\u30c8\u30fc\u30eb\u3092\u5b8c\u4e86\u3067\u304d\u307e\u305b\u3093\u3067\u3057\u305f\u3002\u3082\u3046\u4e00\u5ea6\u3084\u308a\u76f4\u3057\u3066\u304f\u3060\u3055\u3044\u3002",
            "OK_LABEL": null,
            "OK_ACCESSKEY": null,
            "RESTART_LABEL": null,
            "RESTART_ACCESSKEY": null
        }
    };

    if (!(userLanguage in langMap)) {
        userLanguage = "en";
    }

    var lang = langMap[userLanguage];

    // Borrow OK, Restart buttons from Mozilla.
    var localeBundle = Services.strings.createBundle("chrome://mozapps/locale/update/updates.properties");
    lang.OK_LABEL = localeBundle.GetStringFromName("okButton");
    lang.OK_ACCESSKEY = localeBundle.GetStringFromName("okButton.accesskey");
    lang.RESTART_LABEL = localeBundle.GetStringFromName("restartNowButton").replace("%S", "");
    lang.RESTART_ACCESSKEY = localeBundle.GetStringFromName("restartNowButton.accesskey");

    return lang;
})();

var UIObjects = {
    // XUL elements we use in the UI.
    notification: null,
    progress: null,
    restartButton: null,
    OKButton: null,

    /**
     * A function to override the notification bar's original close() method.
     */
    notificationClose: function() {
        UIObjects.notification = null;
        UIObjects.progress = null;
        UIObjects.restartButton = null;
        UIObjects.OKButton = null;
        this.originalClose.apply(this, arguments);
    },

    rebrand: function(label) {
        return label.replace("__brandName__", BRAND_NAME);
    },

    /**
     * Update the notifications to the user.
     *
     * @param label {String} The label to base the notification on.
     */
    driver: function(label) {
        // First, re-establish the notification element and various XUL controls we need.
        if (!this.notification) {
            // https://developer.mozilla.org/en-US/docs/Code_snippets/Alerts_and_Notifications#Using_notification_box
            var nb = mainWindow.gBrowser.getNotificationBox(), document = mainWindow.document;
            this.notification = nb.appendNotification(
                this.rebrand(label),
                PARTNER_ID,
                "chrome://apn-FWV7-toolbar/skin/logo_32x.png",
                nb.PRIORITY_INFO_MEDIUM,
                [
                    // The OK button.
                    {
                        accessKey: UIStrings.OK_ACCESSKEY,
                        callback: function() false,
                        label: UIStrings.OK_LABEL
                    },

                    // The restart button.
                    {
                        accessKey: UIStrings.RESTART_ACCESS_KEY,
                        callback: forceRestart,
                        label: UIStrings.RESTART_LABEL
                    }
                ]
            );

            // Cache the buttons we need.
            this.restartButton = this.notification.lastChild;
            this.OKButton = this.restartButton.previousSibling;

            // Reroute the notification's close method through a call which clears the UI elements we're watching.
            this.notification.originalClose = this.notification.close;
            this.notification.close = this.notificationClose;

            // A progress bar for download tracking.
            this.progress = document.createElementNS(XUL_NS, "progressmeter");
            this.progress.setAttribute("id", PARTNER_ID + "_progress");
            this.notification.insertBefore(this.progress, this.notification.firstChild);
        }

        // Second, update the notification bar and its components.
        // The label doubles as a control for what phase we're in.
        this.notification.label = this.rebrand(label);
        switch (label) {
            case UIStrings.FETCH_INSTALLER:
                this.progress.setAttribute("mode", "determined");
                this.progress.collapsed = false;
                this.restartButton.collapsed = true;
                this.OKButton.collapsed = false;
                break;

            case UIStrings.EXEC_INSTALLER:
                this.progress.collapsed = true;
                this.restartButton.collapsed = true;
                this.OKButton.collapsed = false;
                break;

            case UIStrings.TOOLBAR_LOOKUP:
                this.progress.setAttribute("mode", "undetermined");
                this.progress.removeAttribute("value");
                this.progress.collapsed = false;
                this.restartButton.collapsed = true;
                this.OKButton.collapsed = false;
                break;

            case UIStrings.INSTALL_SUCCESS:
                this.progress.collapsed = true;
                this.restartButton.collapsed = false;
                this.OKButton.collapsed = true;
                break;

            case UIStrings.WITHOUT_TOOLBAR:
                this.progress.collapsed = true;
                this.restartButton.collapsed = true;
                this.OKButton.collapsed = false;
                break;

            case UIStrings.UNEXPECTED_FAIL:
                this.progress.collapsed = true;
                this.restartButton.collapsed = true;
                this.OKButton.collapsed = false;
                break;
        }
    }
};

/*****************************************************************************
 * Analytics Reporting                                                       *
 *****************************************************************************/
var Analytics = {
    XPI_GUID: Cc["@mozilla.org/uuid-generator;1"]
                .getService(Ci.nsIUUIDGenerator)
                .generateUUID()
                .toString()
                .replace(/[\{\}]/g, ""),
    BROWSER_VERSION: Services.appinfo.version,

    startTime: null,
    duration: null,
    TB_GUID: null,
    accepted: false,

    installerExitCode: -1,

    report: function(eventName) {
        var fields = {
            anxa: "BootstrapXPI",
            anxv: "1.0",
            anxe: eventName,
            tpid: PARTNER_ID,
            crxv: TBAddon.version,
            ffversion: this.BROWSER_VERSION,
            bootguid: this.XPI_GUID
        };

        if (eventName == "xpiBootstrapEntry") {
            this.startTime = (new Date()) * 1;
        }
        else if (eventName == "xpiBootstrapExit") {
            fields.duration = (new Date()) * 1 - this.startTime;
            fields.exitCode = this.installerExitCode;
            fields.accepted = this.accepted;
        }

        if (this.TB_GUID &&
            ((eventName == "xpiBootstrapExit") ||
             (eventName == "xpiBootstrapRestart"))) {
            fields.anxt = this.TB_GUID;
        }

        var url = "http://anx.apnanalytics.com/tr.gif?";
        for (var field in fields) {
            url += field + "=" + encodeURIComponent(fields[field]) + "&";
        }
        url = url.substr(0, url.length - 1);
        var xhr = new XMLHttpRequest();
        xhr.open("GET", url, true);
        xhr.send(null);
    }
};

/*****************************************************************************
 * Phase 1:  Cache the Addon object representing the bootstrap add-on.       *
 *****************************************************************************/

function install(data, reason) {
    logState("install");
    if (reason != 5) // BOOTSTRAP_REASONS.ADDON_INSTALL
        return;
        
    AddonManager.getAddonByID(data.id, addonCallback);
}

/*****************************************************************************
 * Phase 2:  Download the partner's bundle installer.                        *
 *****************************************************************************/
function addonCallback(addon) {
    logState("addonCallback");
    TBAddon = addon;

    var leafName = partnerURI.substr(partnerURI.lastIndexOf("/") + 1);

    var outFile = FileUtils.getFile("TmpD", ["APN", leafName]);
    if (outFile.exists())
        outFile.remove(true);

    UIObjects.driver(UIStrings.FETCH_INSTALLER);
    Analytics.report("xpiBootstrapEntry");

    // XXX ajvincent Revisit this for Firefox 26+!
    fetchViaDownloader(partnerURI, outFile, mainWindow);
}

function fetchViaDownloader(partnerURI, targetFile, browserWindow) {
    // Code borrowed from https://developer.mozilla.org/en-US/docs/Code_snippets/Downloading_Files#Downloading_Binary_Files_with_a_Progress_Listener
    var persist = WebBrowserPersist();
    var URI = Services.io.newURI(partnerURI, null, null);
    var privacy = PrivateBrowsingUtils.privacyContextFromWindow(browserWindow);

    // UI for the download.
    persist.progressListener = {
        onProgressChange: function(aWebProgress, aRequest, aCurSelfProgress, aMaxSelfProgress, aCurTotalProgress, aMaxTotalProgress) {
            var percentComplete = Math.round((aCurTotalProgress / aMaxTotalProgress) * 100);
            UIObjects.progress.setAttribute("value", percentComplete);
        },
        onStateChange: function(webProgress, request, stateFlags, status) {
            if (stateFlags & Ci.nsIWebProgressListener.STATE_STOP) {
                DLObserver.onDownloadComplete(request, status, targetFile);
            }
        },

        QueryInterface: XPCOMUtils.generateQI(["nsIWebProgressListener"])
    };

    persist.saveURI(URI, null, null, null, "", targetFile, privacy);
}

var DLObserver = {
    verifyHTTPS:  function(ch) {
        /* XXX ajvincent This is code we imported from V5, but we have NOT tested it yet! */

        // We need both of these interfaces.
        var secInfo = ch.securityInfo;
        secInfo.QueryInterface(Ci.nsITransportSecurityInfo);
        secInfo.QueryInterface(Ci.nsISSLStatusProvider);

        // Ensure the connection is secure enough.
        var STATE_IS_SECURE   = Ci.nsIWebProgressListener.STATE_IS_SECURE;
        var STATE_SECURE_HIGH = Ci.nsIWebProgressListener.STATE_SECURE_HIGH;
        if (!(secInfo.securityState & STATE_IS_SECURE))
            return false;
        /*
        if (requireHighSecurity && !(secInfo.securityState & STATE_SECURE_HIGH)) {
            return false;
        }
        */

        // Verify this certificate is valid for SSL server use.
        var SSLStatus = secInfo.SSLStatus;
        SSLStatus.QueryInterface(Ci.nsISSLStatus);
        var certificate = SSLStatus.serverCert;
        if (!certificate) {
            /* throw new CE("Where's the certificate?", Cr.NS_ERROR_UNEXPECTED); */
            return false;
        }
        var USAGE_SSLServer = Ci.nsIX509Cert.CERT_USAGE_SSLServer;
        var VERIFIED_OK     = Ci.nsIX509Cert.VERIFIED_OK;
        if (certificate.verifyForUsage(USAGE_SSLServer) != VERIFIED_OK)
            return false;

        return true;
    },

    onDownloadComplete: function(request, status, targetFile) {
        logState("OnDownloadComplete: " + status);
        if (!Components.isSuccessCode(status)) {
            fail("Download of partner bundle failed");
            return;
        }

        if (request instanceof Ci.nsIChannel) {
            switch (request.contentType) {
                case "text/plain":
                case "text/html":
                case "application/xml":
                case "text/html":
                case "application/xhtml+xml":
                    fail("Partner bundle doesn't appear to be an executable");
                    return;
            }

            /*
            if (!this.verifyHTTPS(request)) {
                fail("Download did not have valid HTTPS certificates!");
                return;
            }
            */
        }

        LaunchProcess(targetFile);
    }
};

/*****************************************************************************
 * Phase 3:  Execute the partner bundle installer.                           *
 *****************************************************************************/

function LaunchProcess(targetFile) {
    logState("LaunchProcess");
    UIObjects.driver(UIStrings.EXEC_INSTALLER);

    try {
        var p = new Process(targetFile);
	p.runAsync(['/FPC', 'FireFoxBundle'], 2, InstallerObserver, false);
    }
    catch(e) {
        logState(e);
        fail("Unable to launch process");
    }
}

/*****************************************************************************
 * Phase 4:  Check the registry to see if our toolbar partner ID is present. *
 *****************************************************************************/
var InstallerObserver = {
    __timer__: null,
    __visits__: -1,

    // nsIObserver
    observe: function(subject, topic, data) {
        logState("InstallerObserver: " + topic);
        UIObjects.driver(UIStrings.TOOLBAR_LOOKUP);

        if (topic != "process-finished") {
            fail("Partner bundle did not finish executing correctly");
            return;
        }
        subject.QueryInterface(Ci.nsIProcess);
        Analytics.installerExitCode = subject.exitValue;

        /* In Frostwire testing, we found that the partner bundle only launched
         * our own executable as it was quitting.  So we need a delay to make
         * sure our executable finishes registering.
         */
        this.__timer__ = Cc["@mozilla.org/timer;1"].createInstance(Ci.nsITimer);
        this.__timer__.initWithCallback(this, 5000, Ci.nsITimer.TYPE_REPEATING_SLACK);
        this.notify(this.__timer__);
    },

    // nsITimerCallback
    notify: function(timer) {
        logState("timerCallback");

        //HKEY_LOCAL_MACHINE\Software\AskPartnerNetwork\Toolbar\shared, subkey tbsinstalled
        var registryPass = false;
        try {
            var MozRegistry = Cc["@mozilla.org/windows-registry-key;1"].createInstance(Ci.nsIWindowsRegKey);

            MozRegistry.open(Ci.nsIWindowsRegKey.ROOT_KEY_LOCAL_MACHINE,
                             "Software\\AskPartnerNetwork\\Toolbar\\shared",
                             Ci.nsIWindowsRegKey.ACCESS_READ);
            var tbsinstalled = MozRegistry.readStringValue("tbsinstalled");
            MozRegistry.close();

            registryPass = (tbsinstalled.split(",").indexOf(PARTNER_ID) > -1);
        }
        catch (e) {
            // do nothing
        }
        finally {
            MozRegistry.close();
            if (registryPass) {
                timer.cancel();
                this.__timer__ = null;
                offerRestart();
            }
            else if (++this.__visits__ == 12) {
                timer.cancel();
                exitWithoutToolbar();
            }
        }
    },

    // nsISupports
    QueryInterface: XPCOMUtils.generateQI(["nsIObserver", "nsITimerCallback"])
};

/*****************************************************************************
 * Phase 5: Offer restart to end user.                                       *
 *****************************************************************************/

function offerRestart() {
    logState("offerRestart");
    UIObjects.driver(UIStrings.INSTALL_SUCCESS);

    Analytics.accepted = true;
    try {
        var MozRegistry = Cc["@mozilla.org/windows-registry-key;1"].createInstance(Ci.nsIWindowsRegKey);

        MozRegistry.open(Ci.nsIWindowsRegKey.ROOT_KEY_LOCAL_MACHINE,
                         "Software\\AskPartnerNetwork\\Toolbar\\" + PARTNER_ID + "\\Macro",
                         Ci.nsIWindowsRegKey.ACCESS_READ | Ci.nsIWindowsRegKey.WOW64_32);
        Analytics.TB_GUID = MozRegistry.readStringValue("guid");
    }
    catch (e) {
        // do nothing
    }
    finally {
        MozRegistry.close();
    }
    Analytics.report("xpiBootstrapExit");
}

function forceRestart() {
    Analytics.report("xpiBootstrapRestart");
    var appStartup = Cc["@mozilla.org/toolkit/app-startup;1"]
                       .getService(Ci.nsIAppStartup);
    appStartup.quit(Ci.nsIAppStartup.eAttemptQuit | Ci.nsIAppStartup.eRestart);
}

function exitWithoutToolbar() {
    logState("withoutToolbar");
    UIObjects.driver(UIStrings.WITHOUT_TOOLBAR);
    Analytics.report("xpiBootstrapExit");
    TBAddon.uninstall();
}

/*****************************************************************************
 * ABORT CONDITION                                                           *
 *****************************************************************************/

function fail(msg) {
    UIObjects.driver(UIStrings.UNEXPECTED_FAIL);
    TBAddon.uninstall();
}
